import pystan
import matplotlib.pyplot as plt
import numpy as np
import pickle
from pystan import StanModel
from CumNormalDistri import *


K = 4
Data_Noise = [10, 9, 7, 1]
Data_Signal = [2, 1, 19, 5]

Data = {'K': K, 'Freq_Noise': Data_Noise, 'Freq_Signal': Data_Signal}

fit = pystan.stan(file = 'YN_rating.stan', data = Data, iter = 10000, warmup = 1000, seed = 12345, n_jobs = 1)

print(fit)

mu = fit['mu']
# print('mu = \n', mu)
sigma = fit['sigma']
# print('sigma = \n', sigma)
mu_s = np.sort(mu)
sgm_s = np.sort(sigma)
mu_med = mu_s[int(len(mu_s) / 2)]
sgm_med = sgm_s[int(len(sgm_s) / 2)]
print('\nmu_med = ', mu_med)
print('\nsgm_sgm = ', sgm_med)

plt.hist(mu)
plt.title("mu (Delta-m) = {0:<.3f} (med)".format(mu_med))
plt.show()

plt.hist(sigma)
plt.title("Sigma = {0:<.3f} (med)".format(sgm_med))
plt.show()


c = fit['c']

c_comp = []
for i in range(3):
    c_comp.append([])

for v in c:
    for j in range(3):
        c_comp[j].append(v[j])

z_values = np.arange(-4.0, 4.001, 0.1)
x_values = []
y_values = []
for z in z_values:
    x = 1.0 - cum_normal(z)
    x_values.append(x)
    y = 1.0 - cum_normal((z - mu_med) / sgm_med)
    y_values.append(y)

plt.figure(figsize = (5,5))
plt.plot(x_values, y_values, 'b-')
plt.title("ROC-curve")
plt.xlabel("P(False Alarm)")
plt.ylabel("P(Hit)")

cum_freq_n = []
cum_freq_s = []
for k in range(K):
    cum_freq_n.append(Data_Noise[K - 1 - k])
    cum_freq_s.append(Data_Signal[K - 1 - k])

for k in range(1, K):
    cum_freq_n[k] += cum_freq_n[k-1]
    cum_freq_s[k] += cum_freq_s[k-1]

x_fa = []
y_ht = []
for k in range(K-1):
    x_fa.append(cum_freq_n[k] / cum_freq_n[K-1])
    y_ht.append(cum_freq_s[k] / cum_freq_s[K-1])
plt.plot(x_fa, y_ht, 'ro')

c_s = []
c_med = []
for k in range(K-1):
    c_s.append([])
    c_s[k] = np.sort(c_comp[k])
    c_med.append(c_s[k][int(len(c_s[k]) / 2)])

print('\nCriterions = ')
for k in range(K-1):
    print('c-{0} = {1:<.3f} (med)'.format(k + 1, c_med[k]))

    
x_fa = []
y_ht = []

for k in range(K-1):
    x_fa.append(1.0 - cum_normal(c_med[k]))
    y_ht.append(1.0 - cum_normal((c_med[k] - mu_med) / sgm_med))
plt.plot(x_fa, y_ht, 'gs')

plt.show()

n_smpl = len(c_s[0])

min_c = c_s[0][0]
max_c = c_s[0][0]
for k in range(K-1):
    for i in range(n_smpl):
        if min_c > c_s[k][i]:
            min_c = c_s[k][i]
        if max_c < c_s[k][i]:
            max_c = c_s[k][i]

c_cnt = []
for k in range(K-1):
    c_cnt.append([])
    for i in range(20):
        c_cnt[k].append(0)
    for i in range(n_smpl):
        pos = 20.0 * (c_s[k][i] - min_c) / (max_c - min_c)
        pos = int(pos) if pos < 20.0 else 19
        c_cnt[k][pos] += 1

x_v = [0]*20
for i in range(20):
    x_v[i] = min_c + (max_c - min_c) * (i + 0.5) / 20.0

for k in range(K-1):
    plt.plot(x_v, c_cnt[k], 'b-')
title_str = "Criterions (medians)\nc{0}({1:<.2f})".format(1, c_med[0])
for k in range(1, K-1):
    title_str += "  c{0}({1:<.2f})".format(k+1, c_med[k])
plt.title(title_str)

plt.show()
